import copy

class TaintException(Exception):
    pass

class Policy(object):
    '''
    top is the top element (the taint (or taints) applicable for untrusted data)
    bottom is the bottom element (the taint (or taints) applicable for completed trusted data, like constants)
    join is the join operation. I have two piece of data to mix, how I should comute the tag of the result.
    downgrader is how to clean a piece of data
    has. is a tag the LatticeElement?
    type. Consructor type for the elements
    cleaners. Which tag clean each filter {filter: tag}
    alarm (optional)
    '''
    def __init__(self,initial_state=None):
        if initial_state == None: self.state=copy.copy(self.bottom)
        else: self.state=self.type(initial_state)

    def alarm(self,ssink,expected,details):
	text='Danger data reach the sink "%s" (sensible to %s). Details:' % (ssink,expected)
	for (ss,t) in details.iteritems(): text+='\n\tsubscript \'%s\' is tainted with %s' % (ss,t)
	raise TaintException(text)

    def __repr__(self):
	return str(self.state)

    def __str__(self):
	return str(self.__repr__())

    @property
    def isBottom(self):
        return self.state == self.bottom

class SimpleTags(Policy):
    keys=set(range(1,5))
    top=keys
    bottom=set()
    type=set
    cleaners={
            'escape':1, # 1 means XSS
    }
    ssinks={
	'middleware': 1, # 1 means XSS
	}

    def join(self,aTagSet):
        self.state.update(aTagSet.state)

    def downgrader(self,aTag):
        self.state.discard(aTag)

    def has(self,aTag):
        return aTag in self.state

    def cmp_le(self,aTagSet):
	#TODO, is this working?
        left=self.state
        right=aTagSet.state
        if len(left) == 1 and left.subset(right): return True
        else: raise NotImplementedError

Comp='Compositional'
NonComp='NonCompositional'

class SanitizersOrder(Policy):
    filters={
        'escape':'',
    }
    top=[]
    bottom='trusworthy'
    type=list
    cleaners={
	'escape':'escape',
	'striptags':'striptags',
	'urlize':'urlize',
	'shorturl':'shorturl',
	}
    ssinks={
	'middleware': ['escape','urlize','shorturl'],
    }
    sanitizers={
	'escape'  : Comp,
	'urlize'  : NonComp,
	'shorturl': NonComp,
	}

    def prefixModNonCompositionalSanitizers(self,array1,array2,NonCompositional=True):
	'''are NonCompositional sanitizer important? default=True'''
        index=0
        if array1==self.bottom and array2==self.bottom: return self.bottom
        if array1==self.bottom or array2==self.bottom: return []
        for index in range(min(len(array1),len(array2))):
              a=array1[index]
              b=array2[index]
              if a!=b or (NonCompositional and self.sanitizers[b]==NonComp):
                  index-=1
                  break
        return array1[:index+1]


    def join(self,aTagSet):
	self.state=self.prefixModNonCompositionalSanitizers(self.state,aTagSet.state)

    def safe_zone(self,level):
	'''return true if self is in the safe zone defined by level.'''
        if self.isBottom: return True
	return self.prefixModNonCompositionalSanitizers(self.state,level,False)==level
	
    def has(self,aTag):
        if self.isBottom: return False
        return not self.state[:len(aTag)]==aTag

    def downgrader(self,aTag):
        if self.isBottom: return
	if hasattr(aTag,'__iter__'): self.state.extend(aTag)
        else: self.state.append(aTag)

    def alarm(self,ssink,expected,details):
	text='wrong sequence of sanitizers '
	if len(details)==1:
		(ss,t)=details.popitem()
		text+='%s:[\'%s\']' % (t,ss)
	else:
		for (ss,t) in details.iteritems(): text+='\n\t%s: %s' % (t,ss)
	raise TaintException(text)
