# -*- coding: utf-8 -*-
'''
Taint Mode for Python via a Library


Copyright 2009 Juan José Conti
Copyright 2010 Juan José Conti - Alejandro Russo

This file is part of taintmode.py

taintmode is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

taintmode.py is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with taintmode.py.  If not, see <http://www.gnu.org/licenses/>.

'''
from taintmode import taint,DtStringIO
from StringIO import StringIO

def getinput(environ): #Mostly copied from http://pypi.python.org/pypi/wsgiform/0.3
   '''Non-destructively retrieves wsgi.input value.'''
   wsginput = environ['wsgi.input']
   # Non-destructively fetch string value of wsgi.input
   if hasattr(wsginput, 'getvalue'):
       qs = wsginput.getvalue()
   # Otherwise, read and reconstruct wsgi.input
   else:
       # Never read more than content length
       clength = int(environ['CONTENT_LENGTH'])
       qs = wsginput.read(clength)
       environ['wsgi.input'] = StringIO(qs)
   return qs

def initRESPONSEwrapper(initMethod):
    def myinit(self,*args, **kargs):
        '''webapp.WSGIApplication.RESPONSE_CLASS constructs a
        response with the default settings. Among them, an
        instance attribute called out, which is a StringIO by
        default. https://code.google.com/appengine/docs/python/tools/webapp/responseclass.html#Response_out
        We need to support taints on that.'''
        initMethod(self,*args, **kargs)
        self.out=DtStringIO()
    return myinit

class TaintMiddleware(object):
    def __init__(self,app):
        self.app=app
        self.app.RESPONSE_CLASS.__init__=initRESPONSEwrapper(self.app.RESPONSE_CLASS.__init__)

    def __call__(self,environ,start_response):
        environ=taint(environ)
        def my_start_response(status, response_headers, exc_info=None):
          write = start_response(status, response_headers, exc_info)
          def my_write(body_data):
	    body_data.checkAndAlarm('middleware')
            write(body_data)
          return my_write
        return self.app(environ, my_start_response)
