# -*- coding: utf-8 -*-
'''
Taint Mode for Python via a Library


Copyright 2009 Juan José Conti
Copyright 2010 Juan José Conti - Alejandro Russo

This file is part of taintmode.py

taintmode is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

taintmode.py is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with taintmode.py.  If not, see <http://www.gnu.org/licenses/>.

'''
from taintmode import remove_tags,DtString,taint_aware,collect_tags
import taintConfig

__all__ = [ 'templateWrapper' ]

def collapse_tags(listOftags):
    '''given a list of tags, return a Policy with the join of all of them'''
    t = taintConfig.Policy()
    for i in listOftags:
        t.join(i)
    return t

def templateWrapper(originalModule):
    import taintConfig
    def VariableNodeRender(originalRender):
        def inner (*args, **kwargs):
            node=args[0]
            ctx=args[1]
            tags=collapse_tags([ collect_tags(i) for i in ctx.dicts[0].itervalues()])
            r  = originalRender(*args, **kwargs)
            throughFilters=[]
            for filter in node.filter_expression.filters:
                if hasattr(filter[0],'_decorated_function'): #the filter is decorated
                    filterName=filter[0]._decorated_function.func_name
		else:
                    filterName=filter[0].func_name
                if filterName in taintConfig.Policy.cleaners:
		   throughFilters.append(taintConfig.Policy.cleaners[filterName])
            r=taint_aware(r,tags)
            remove_tags(r,throughFilters)
            return r
        return inner

    def NodeListRender(originalRender):
        def inner(self, context):
            bits = []
            for node in self:
                if isinstance(node, originalModule.django.template.Node):
                    bits.append(self.render_node(node, context))
                else:
                    bits.append(node)
            return DtString('').join(bits)
        return inner

    originalModule.django.template.VariableNode.render=VariableNodeRender(originalModule.django.template.VariableNode.render)
    originalModule.django.template.NodeList.render=NodeListRender(originalModule.django.template.NodeList.render)
    return originalModule
